#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import json
import logging

from flask import Blueprint, jsonify, request, Response
from oceandb_driver_interface.search_model import FullTextModel, QueryModel
from plecos.plecos import (
    is_valid_dict_local,
    list_errors_dict_local,
)

from aquarius.app.dao import Dao
from aquarius.app.util import (
    make_paginate_response,
    datetime_converter,
    get_metadata_from_services,
    sanitize_record,
    list_errors,
)
from aquarius.log import setup_logging
from aquarius.myapp import app

setup_logging()
assets = Blueprint('assets', __name__)

# Prepare OceanDB
dao = Dao(config_file=app.config['CONFIG_FILE'])
logger = logging.getLogger('aquarius')


@assets.route('', methods=['GET'])
def get_assets_ids():
    """Get all asset IDs.
    ---
    tags:
      - ddo
    responses:
      200:
        description: successful action
    """
    asset_with_id = dao.get_all_listed_assets()
    asset_ids = [a['id'] for a in asset_with_id]
    resp_body = dict({'ids': asset_ids})
    return Response(sanitize_record(resp_body), 200, content_type='application/json')


@assets.route('/ddo/<did>', methods=['GET'])
def get_ddo(did):
    """Get DDO of a particular asset.
    ---
    tags:
      - ddo
    parameters:
      - name: did
        in: path
        description: DID of the asset.
        required: true
        type: string
    responses:
      200:
        description: successful operation
      404:
        description: This asset DID is not in OceanDB
    """
    try:
        asset_record = dao.get(did)
        return Response(sanitize_record(asset_record), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'get_ddo: {str(e)}')
        return f'{did} asset DID is not in OceanDB', 404


@assets.route('/ddo', methods=['GET'])
def get_asset_ddos():
    """Get DDO of all assets.
    ---
    tags:
      - ddo
    responses:
      200:
        description: successful action
    """
    assets_with_id = dao.get_all_listed_assets()
    assets_metadata = {a['id']: a for a in assets_with_id}
    for _, _record in assets_metadata.items():
        sanitize_record(_record)
    return Response(json.dumps(assets_metadata, default=datetime_converter), 200,
                    content_type='application/json')


@assets.route('/metadata/<did>', methods=['GET'])
def get_metadata(did):
    """Get metadata of a particular asset
    ---
    tags:
      - metadata
    parameters:
      - name: did
        in: path
        description: DID of the asset.
        required: true
        type: string
    responses:
      200:
        description: successful operation.
      404:
        description: This asset DID is not in OceanDB.
    """
    try:
        asset_record = dao.get(did)
        metadata = get_metadata_from_services(asset_record['service'])
        return Response(sanitize_record(metadata), 200, content_type='application/json')
    except Exception as e:
        logger.error(f'get_metadata: {str(e)}')
        return f'{did} asset DID is not in OceanDB', 404


###########################
# SEARCH
###########################

@assets.route('/ddo/query', methods=['GET'])
def query_text():
    """Get a list of DDOs that match with the given text.
    ---
    tags:
      - ddo
    parameters:
      - name: text
        in: query
        description: ID of the asset.
        required: true
        type: string
      - name: sort
        in: query
        type: object
        description: Key or list of keys to sort the result
        example: {"value":1}
      - name: offset
        in: query
        type: int
        description: Number of records per page
        example: 100
      - name: page
        in: query
        type: int
        description: Page showed
        example: 1
    responses:
      200:
        description: successful action
    """
    data = request.args
    assert isinstance(
        data, dict), 'invalid `args` type, should already formatted into a dict.'
    search_model = FullTextModel(text=data.get('text', None),
                                 sort=None if data.get('sort', None) is None else json.loads(
                                     data.get('sort', None)),
                                 offset=int(data.get('offset', 100)),
                                 page=int(data.get('page', 1)))
    query_result = dao.query(search_model)
    for i in query_result[0]:
        sanitize_record(i)

    response = make_paginate_response(query_result, search_model)
    return Response(json.dumps(response, default=datetime_converter), 200,
                    content_type='application/json')


@assets.route('/ddo/query', methods=['POST'])
def query_ddo():
    """Get a list of DDOs that match with the executed query.
    ---
    tags:
      - ddo
    consumes:
      - application/json
    parameters:
      - in: body
        name: body
        required: true
        description: Asset metadata.
        schema:
          type: object
          properties:
            query:
              type: string
              description: Query to realize
              example: {"value":1}
            sort:
              type: object
              description: Key or list of keys to sort the result
              example: {"value":1}
            offset:
              type: int
              description: Number of records per page
              example: 100
            page:
              type: int
              description: Page showed
              example: 1
    responses:
      200:
        description: successful action
    """
    assert isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    assert isinstance(
        data, dict), 'invalid `body` type, should be formatted as a dict.'
    if 'query' in data:
        search_model = QueryModel(query=data.get('query'), sort=data.get('sort'),
                                  offset=data.get('offset', 100),
                                  page=data.get('page', 1))
    else:
        search_model = QueryModel(query={}, sort=data.get('sort'),
                                  offset=data.get('offset', 100),
                                  page=data.get('page', 1))
    query_result = dao.query(search_model)
    for ddo in query_result[0]:
        sanitize_record(ddo)

    response = make_paginate_response(query_result, search_model)
    return Response(json.dumps(response, default=datetime_converter), 200,
                    content_type='application/json')


###########################
# VALIDATE
###########################

@assets.route('/ddo/validate', methods=['POST'])
def validate():
    """Validate metadata content.
    ---
    tags:
      - ddo
    consumes:
      - application/json
    parameters:
      - in: body
        name: body
        required: true
        description: Asset metadata.
        schema:
          type: object
    responses:
      200:
        description: successfully request.
      500:
        description: Error
    """
    assert isinstance(request.json, dict), 'invalid payload format.'
    data = request.json
    assert isinstance(
        data, dict), 'invalid `body` type, should be formatted as a dict.'

    if is_valid_dict_local(data):
        return jsonify(True)
    else:
        res = jsonify(list_errors(list_errors_dict_local, data))
        return res



json_dict = {
  "@context": "https://w3id.org/did/v1",
  "id": "did:op:0c184915b07b44c888d468be85a9b28253e80070e5294b1aaed81c2f0264e430",
  "dataToken": "0xC7EC1970B09224B317c52d92f37F5e1E4fF6B687",
  "created": "2019-05-22T08:44:27Z",
  "publicKey": [
    {
      "id": "did:op:0c184915b07b44c888d468be85a9b28253e80070e5294b1aaed81c2f0264e430",
      "type": "EthereumECDSAKey",
      "owner": "0x00Bd138aBD70e2F00903268F3Db08f2D25677C9e"
    }
  ],
  "authentication": [
    {
      "type": "RsaSignatureAuthentication2018",
      "publicKey": "did:op:0c184915b07b44c888d468be85a9b28253e80070e5294b1aaed81c2f0264e430"
    }
  ],
  "service": [
    {
      "type": "access",
      "serviceEndpoint": "http://localhost:8030/api/v1/brizo/services/consume",
      "purchaseEndpoint": "http://localhost:8030/api/v1/brizo/services/access/initialize",
      "index": 1,
      "attributes": {
                "main": {
                    "cost":"10",
                    "timeout":"0",
                    "name": "dataAssetAccessServiceAgreement"
                }
      }
    },
    {
      "type": "metadata",
      "serviceEndpoint": "http://localhost:5000/api/v1/aquarius/assets/ddo/did:op:0c184915b07b44c888d468be85a9b28253e80070e5294b1aaed81c2f0264e430",
      "attributes": {
        "main": {
          "name": "Ocean protocol white paper",
          "type": "dataset",
          "dateCreated": "2012-10-10T17:00:00Z",
          "datePublished": "2012-10-10T17:00:00Z",
          "author": "Ocean Protocol Foundation Ltd.",
          "license": "CC-BY",
          "files": [
            {
              "checksum": "efb2c764274b745f5fc37f97c6b0e761",
              "contentType": "text/csv",
              "checksumType": "MD5",
              "contentLength": "4535431",
              "resourceId": "access-log2018-02-13-15-17-29-18386C502CAEA932",
              "index": 0
            },
            {
              "checksum": "efb2c764274b745f5fc37f97c6b0e761",
              "contentType": "text/csv",

              "contentLength": "4535431",
              "resourceId": "access-log2018-02-13-15-17-29-18386C502CAEA932",
              "index": 1
            },
            {
              "index": 2,
              "contentType": "text/csv",

            }
          ]
        },
        "encryptedFiles": "<tests.resources.mocks.secret_store_mock.SecretStoreMock object at 0x7f8146a94710>.0c184915b07b44c888d468be85a9b28253e80070e5294b1aaed81c2f0264e430!![{\"url\": \"https://testocnfiles.blob.core.windows.net/testfiles/testzkp.pdf\", \"checksum\": \"efb2c764274b745f5fc37f97c6b0e761\", \"checksumType\": \"MD5\", \"contentLength\": \"4535431\", \"resourceId\": \"access-log2018-02-13-15-17-29-18386C502CAEA932\"}, {\"url\": \"s3://ocean-test-osmosis-data-plugin-dataseeding-1537375953/data.txt\", \"checksum\": \"efb2c764274b745f5fc37f97c6b0e761\", \"contentLength\": \"4535431\", \"resourceId\": \"access-log2018-02-13-15-17-29-18386C502CAEA932\"}, {\"url\": \"http://ipv4.download.thinkbroadband.com/5MB.zip\"}]!!0",
        "curation": {
          "rating": 0.93,
          "numVotes": 123,
          "schema": "Binary Voting"
        },
        "additionalInformation": {
          "description": "Introduce the main concepts and vision behind ocean protocol",
          "copyrightHolder": "Ocean Protocol Foundation Ltd.",
          "workExample": "Text PDF",
          "inLanguage": "en",
          "categories": [
            "white-papers"
          ],
          "tags": ["data exchange", "sharing", "curation", "bonding curve"],
          "links": [
            {
              "url": "http://data.ceda.ac.uk/badc/ukcp09/data/gridded-land-obs/gridded-land-obs"
                     "-daily/"
            },
            {
              "url": "http://data.ceda.ac.uk/badc/ukcp09/data/gridded-land-obs/gridded-land-obs"
                     "-averages-25km/"
            },
            {
              "url": "http://data.ceda.ac.uk/badc/ukcp09/"
            }
          ],
          "updateFrequency": "yearly",
          "structuredMarkup": [
            {
              "uri": "http://skos.um.es/unescothes/C01194/jsonld",
              "mediaType": "application/ld+json"
            },
            {
              "uri": "http://skos.um.es/unescothes/C01194/turtle",
              "mediaType": "text/turtle"
            }
          ]
        }
      },
      "index": 2
    }
  ],
  "proof": {
    "type": "DDOIntegritySignature",
    "created": "2019-05-22T08:44:27Z",
    "creator": "0x00Bd138aBD70e2F00903268F3Db08f2D25677C9e",
    "signatureValue": "0xbd7b46b3ac664167bc70ac211b1a1da0baed9ead91613a5f02dfc25c1bb6e3ff40861b455017e8a587fd4e37b703436072598c3a81ec88be28bfe33b61554a471b"
  }
}



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
import os

import pytest
from ocean_lib.config import Config
from ocean_lib.config_provider import ConfigProvider
from ocean_lib.web3_internal.contract_handler import ContractHandler
from ocean_lib.web3_internal.web3_provider import Web3Provider

from aquarius.events.events_monitor import EventsMonitor
from aquarius.constants import BaseURLs
from aquarius.events.util import get_artifacts_path
from aquarius.run import app

app = app

EVENTS_INSTANCE = None


@pytest.fixture
def base_ddo_url():
    return BaseURLs.BASE_AQUARIUS_URL + '/assets/ddo'


@pytest.fixture
def client_with_no_data():
    client = app.test_client()
    client.delete(BaseURLs.BASE_AQUARIUS_URL + '/assets/ddo')
    yield client


@pytest.fixture
def client():
    client = app.test_client()

    yield client


@pytest.fixture
def events_object():
    global EVENTS_INSTANCE
    if not EVENTS_INSTANCE:
        config_file = os.getenv('CONFIG_FILE', 'config.ini')
        network_rpc = os.environ.get('EVENTS_RPC', 'http://127.0.0.1:8545')

        ConfigProvider.set_config(Config(config_file))
        from ocean_lib.ocean.util import get_web3_connection_provider

        Web3Provider.init_web3(provider=get_web3_connection_provider(network_rpc))
        ContractHandler.set_artifacts_path(get_artifacts_path())

        EVENTS_INSTANCE = EventsMonitor(
            Web3Provider.get_web3(),
            app.config['CONFIG_FILE']
        )
        EVENTS_INSTANCE.store_last_processed_block(0)
    return EVENTS_INSTANCE



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import logging
import logging.config
import os

import coloredlogs
import yaml


def setup_logging(default_path='logging.yaml', default_level=None, env_key='LOG_CFG'):
    """Logging Setup"""
    path = default_path
    value = os.getenv(env_key, None)
    if value:
        path = value

    log_level_name = os.getenv('LOG_LEVEL', None)

    if not default_level:
        level_map = {
            'INFO': logging.INFO,
            'DEBUG': logging.DEBUG,
            'WARNING': logging.WARNING,
            'ERROR': logging.ERROR
        }
        default_level = logging.INFO
        if log_level_name:
            default_level = level_map.get(log_level_name, logging.INFO)

    print(f'default log level: {default_level}, env var LOG_LEVEL {os.getenv("LOG_LEVEL", "NOT SET")}')

    if os.path.exists(path):
        with open(path, 'rt') as f:
            try:
                config = yaml.safe_load(f.read())
                if log_level_name:
                    for logger in config['loggers']:
                        if logger == 'elasticsearch':
                            continue

                        config['loggers'][logger]['level'] = log_level_name
                logging.config.dictConfig(config)
                coloredlogs.install()
            except Exception as e:
                print(f'Error in Logging Configuration (using default configs): {e}')
                logging.basicConfig(level=default_level)
                coloredlogs.install(level=default_level)
    else:
        logging.basicConfig(level=default_level)
        coloredlogs.install(level=default_level)
        print('Failed to load configuration file. Using default configs')



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
import json
import lzma

import ecies
from web3 import Web3

import eth_keys

from aquarius.events.constants import EVENT_METADATA_CREATED, EVENT_METADATA_UPDATED
from tests.helpers import (
    get_web3,
    new_ddo,
    test_account1,
    test_account3,
    send_create_update_tx,
    ecies_account,
    get_ddo,
    get_event,
)


def run_test(client, base_ddo_url, events_instance, flags=None, encryption_key=None):
    web3 = get_web3()
    block = web3.eth.blockNumber
    _ddo = new_ddo(test_account1, web3, f'dt.{block}')
    did = _ddo.id
    ddo_string = json.dumps(dict(_ddo))
    data = Web3.toBytes(text=ddo_string)
    _flags = flags or 0
    if flags is not None:
        data = lzma.compress(data)
        # mark bit 1
        _flags = _flags | 1

    if encryption_key is not None:
        # ecies encrypt - bit 2
        _flags = _flags | 2
        key = eth_keys.KeyAPI.PrivateKey(encryption_key)
        data = ecies.encrypt(key.public_key.to_hex(), data)

    send_create_update_tx('create', did, bytes([_flags]), data, test_account1)
    get_event(EVENT_METADATA_CREATED, block, did)
    events_instance.process_current_blocks()
    published_ddo = get_ddo(client, base_ddo_url, did)
    assert published_ddo['id'] == did

    _ddo['service'][0]['attributes']['main']['name'] = 'Updated ddo by event'
    ddo_string = json.dumps(dict(_ddo))
    data = Web3.toBytes(text=ddo_string)
    if flags is not None:
        data = lzma.compress(data)

    if encryption_key is not None:
        key = eth_keys.KeyAPI.PrivateKey(encryption_key)
        data = ecies.encrypt(key.public_key.to_hex(), data)

    send_create_update_tx('update', did, bytes([_flags]), data, test_account1)
    get_event(EVENT_METADATA_UPDATED, block, did)
    events_instance.process_current_blocks()
    published_ddo = get_ddo(client, base_ddo_url, did)
    assert published_ddo['id'] == did
    assert published_ddo['service'][0]['attributes']['main']['name'] == 'Updated ddo by event'


def test_publish_and_update_ddo(client, base_ddo_url, events_object):
    run_test(client, base_ddo_url, events_object)


def test_publish_and_update_ddo_with_lzma(client, base_ddo_url, events_object):
    run_test(client, base_ddo_url, events_object, 0)


def test_publish_and_update_ddo_with_lzma_and_ecies(client, base_ddo_url, events_object):
    run_test(client, base_ddo_url, events_object, 0, ecies_account.privateKey)


def test_publish(client, base_ddo_url, events_object):
    _ddo = new_ddo(test_account1, get_web3(), f'dt.0')
    did = _ddo.id
    ddo_string = json.dumps(dict(_ddo))
    data = Web3.toBytes(text=ddo_string)
    send_create_update_tx('create', did, bytes([0]), data, test_account1)
    events_object.process_current_blocks()
    published_ddo = get_ddo(client, base_ddo_url, did)
    assert published_ddo['id'] == did


def test_publish_unallowed_address(client, base_ddo_url, events_object):
    _ddo = new_ddo(test_account3, get_web3(), f'dt.0')
    did = _ddo.id
    ddo_string = json.dumps(dict(_ddo))
    data = Web3.toBytes(text=ddo_string)
    send_create_update_tx('create', did, bytes([0]), data, test_account3)
    events_object.process_current_blocks()
    published_ddo = get_ddo(client, base_ddo_url, did)
    assert published_ddo == None



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
import json
import lzma
import os
import uuid

from web3 import Web3

from aquarius.app.dao import Dao
from aquarius.app.util import validate_date_format
from aquarius.events.constants import EVENT_METADATA_CREATED
from aquarius.run import get_status, get_version
from tests.ddo_samples_invalid import json_dict_no_valid_metadata
from tests.ddos.ddo_sample1 import json_dict
from tests.ddos.ddo_sample_updates import json_before, json_valid
from eth_account.messages import defunct_hash_message

from tests.helpers import new_ddo, test_account1, send_create_update_tx, get_event, get_web3


def sign_message(account, message_str):
    msg_hash = defunct_hash_message(text=message_str)
    full_signature = account.sign_message(msg_hash)
    return full_signature.signature.hex()


def get_ddo(client, base_ddo_url, did):
    rv = client.get(
        base_ddo_url + f'/{did}',
        content_type='application/json'
    )
    fetched_ddo = json.loads(rv.data.decode('utf-8'))
    return fetched_ddo


def run_request_get_data(client_method, url, data=None):
    _response = run_request(client_method, url, data)
    print(f'response: {_response}')
    if _response and _response.data:
        return json.loads(_response.data.decode('utf-8'))

    return None


def run_request(client_method, url, data=None):
    if data is None:
        _response = client_method(url, content_type='application/json')
    else:
        _response = client_method(
            url, data=json.dumps(data), content_type='application/json'
        )

    return _response


def test_version(client):
    """Test version in root endpoint"""
    rv = client.get('/')
    assert json.loads(rv.data.decode('utf-8'))['software'] == 'Aquarius'
    assert json.loads(rv.data.decode('utf-8'))['version'] == get_version()


def test_health(client):
    """Test health check endpoint"""
    rv = client.get('/health')
    assert rv.data.decode('utf-8') == get_status()[0]


def test_post_with_no_valid_ddo(client, base_ddo_url, events_object):
    block = get_web3().eth.blockNumber
    ddo = new_ddo(test_account1, get_web3(), f'dt.{block}', json_dict_no_valid_metadata)
    ddo_string = json.dumps(dict(ddo.items()))
    _receipt = send_create_update_tx(
        'create', ddo.id, bytes([1]), lzma.compress(Web3.toBytes(text=ddo_string)), test_account1)
    get_event(EVENT_METADATA_CREATED, block, ddo.id)
    events_object.process_current_blocks()
    try:
        published_ddo = get_ddo(client, base_ddo_url, ddo.id)
        assert not published_ddo, f'publish should fail, Aquarius validation ' \
                                  f'should have failed and skipped the {EVENT_METADATA_CREATED} event.'
    except Exception:
        pass


def test_query_metadata(client, base_ddo_url, events_object):
    dao = Dao(config_file=os.environ['CONFIG_FILE'])
    dao.delete_all()

    block = get_web3().eth.blockNumber
    assets = []
    txs = []
    for i in range(5):
        ddo = new_ddo(test_account1, get_web3(), f'dt.{i+block}', json_dict)
        assets.append(ddo)

        txs.append(
            send_create_update_tx(
                'create', ddo.id,
                bytes([1]),
                lzma.compress(Web3.toBytes(text=json.dumps(dict(ddo.items())))),
                test_account1
            )
        )

    for ddo in assets:
        get_event(EVENT_METADATA_CREATED, block, ddo.id)
        events_object.process_current_blocks()

    num_assets = len(assets)

    offset = 2
    response = run_request_get_data(
        client.get, base_ddo_url + f'/query?text=white&page=1&offset={offset}')
    assert response['page'] == 1
    assert response['total_pages'] == int(num_assets / offset) + int(num_assets % offset > 0)
    assert response['total_results'] == num_assets
    assert len(response['results']) == offset

    response = run_request_get_data(
        client.get, base_ddo_url + f'/query?text=white&page=3&offset={offset}')
    assert response['page'] == 3
    assert response['total_pages'] == int(
        num_assets / offset) + int(num_assets % offset > 0)
    assert response['total_results'] == num_assets
    assert len(response['results']) == num_assets - \
        (offset * (response['total_pages'] - 1))

    response = run_request_get_data(
        client.get, base_ddo_url + f'/query?text=white&page=4&offset={offset}')
    assert response['page'] == 4
    assert response['total_pages'] == int(
        num_assets / offset) + int(num_assets % offset > 0)
    assert response['total_results'] == num_assets
    assert len(response['results']) == 0


def test_validate(client_with_no_data, base_ddo_url):
    post = run_request(
        client_with_no_data.post,
        base_ddo_url + '/validate', data={}
    )
    assert post.status_code == 200
    assert post.data == b'[{"message":"\'main\' is a required property","path":""}]\n'
    post = run_request(
        client_with_no_data.post,
        base_ddo_url + '/validate',
        data=json_valid
    )
    assert post.data == b'true\n'


def test_date_format_validator():
    date = '2016-02-08T16:02:20Z'
    assert validate_date_format(date) == (None, None)


def test_invalid_date():
    date = 'XXXX'
    assert validate_date_format(date) == (
        "Incorrect data format, should be '%Y-%m-%dT%H:%M:%SZ'", 400)


def test_resolveByDtAddress(client_with_no_data, base_ddo_url, events_object):
    client = client_with_no_data
    block = get_web3().eth.blockNumber
    _ddo = json_before.copy()
    ddo = new_ddo(test_account1, get_web3(), f'dt.{block}', _ddo)
    send_create_update_tx(
        'create',
        ddo['id'],
        bytes([1]),
        lzma.compress(Web3.toBytes(text=json.dumps(dict(ddo)))),
        test_account1
    )
    get_event(EVENT_METADATA_CREATED, block, ddo['id'])
    events_object.process_current_blocks()
    assert len(
        run_request_get_data(client.post, base_ddo_url + '/query',
                             {"query": {"dataToken": [_ddo['dataToken']]}}
                             )['results']
    ) > 0



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0

import logging

import elasticsearch
from oceandb_driver_interface import OceanDb
from oceandb_driver_interface.search_model import FullTextModel, QueryModel


class Dao(object):

    def __init__(self, oceandb=None, config_file=None):
        self.oceandb = oceandb or OceanDb(config_file).plugin

    def get_all_listed_assets(self):
        assets = self.oceandb.list()
        asset_with_id = []
        for asset in assets:
            try:
                if len(asset) == 1:
                    continue

                if self.is_listed(asset['service']):
                    asset_with_id.append(asset)
            except Exception as e:
                logging.error(f'Dao.get_all_listed_assets: {str(e)}')

        return asset_with_id

    def get_all_assets(self):
        return [a for a in self.oceandb.list()]

    def get(self, asset_id):
        try:
            asset = self.oceandb.read(asset_id)
        except elasticsearch.exceptions.NotFoundError as e:
            logging.error(f'Dao.get -- asset with id {asset_id} was not found, original error was:{str(e)}')
            raise

        except Exception as e:
            logging.error(f'Dao.get: {str(e)}')
            raise

        if asset is None or not self.is_listed(asset['service']):
            return None

        return asset

    def register(self, record, asset_id):
        return self.oceandb.write(record, asset_id)

    def update(self, record, asset_id):
        return self.oceandb.update(record, asset_id)

    def delete(self, asset_id):
        return self.oceandb.delete(asset_id)

    def delete_all(self):
        if hasattr(self.oceandb, 'delete_all'):
            self.oceandb.delete_all()
        else:
            assets = self.oceandb.list()
            for asset in assets:
                try:
                    self.delete(asset['id'])
                except Exception as e:
                    logging.error(f'Dao.delete_all: {str(e)}')

    def query(self, query):
        query_list = []
        if isinstance(query, QueryModel):
            query_result, count = self.oceandb.query(query)
        elif isinstance(query, FullTextModel):
            query_result, count = self.oceandb.text_query(query)
        else:
            raise TypeError('Unrecognized `query` type %s' % type(query))

        for f in query_result:
            if 'service' in f:
                if self.is_listed(f['service']):
                    query_list.append(f)

        return query_list, count

    @staticmethod
    def is_listed(services):
        for service in services:
            if service['type'] == 'metadata':
                if 'curation' in service['attributes'] and 'isListed' in service['attributes']['curation']:
                    return service['attributes']['curation']['isListed']



json_before = {
    "@context": "https://w3id.org/future-method/v1",
    "created": "2016-02-08T16:02:20Z",
    "dataToken": "0xC7EC1970B09224B317c52d92f37F5e1E4fF6B687",
    "id": "did:op:112233445566778899",
    "publicKey": [
        {
            "id": "did:op:123456789abcdefghi#keys-1",
            "type": "RsaVerificationKey2018",
            "owner": "did:op:123456789abcdefghi",
            "publicKeyPem": "-----BEGIN PUBLIC KEY...END PUBLIC KEY-----\r\n"
        },
        {
            "id": "did:op:123456789abcdefghi#keys-2",
            "type": "Ed25519VerificationKey2018",
            "owner": "did:op:123456789abcdefghi",
            "publicKeyBase58": "H3C2AVvLMv6gmMNam3uVAjZpfkcJCwDwnZn6z3wXmqPV"
        }
    ],
    "authentication": [
        {
            "type": "RsaSignatureAuthentication2018",
            "publicKey": "did:op:123456789abcdefghi#keys-1"
        },
        {
            "type": "ieee2410Authentication2018",
            "publicKey": "did:op:123456789abcdefghi#keys-2"
        }
    ],
    "proof": {
        "type": "UUIDSignature",
        "created": "2016-02-08T16:02:20Z",
        "creator": "did:example:8uQhQMGzWxR8vw5P3UWH1ja",
        "signatureValue": "QNB13Y7Q9...1tzjn4w=="
    },
    "service": [
        {
            "attributes": {
                "main": {
                    "cost":"10",
                    "timeout":"0"
                }
            },
            "type": "access",
            "index": 0,
            "serviceEndpoint": "http://mybrizo.org/api/v1/brizo/services/consume?pubKey=${"
                               "pubKey}&serviceId={serviceId}&url={url}"
        },
        {
            "attributes": {
                "main": {
                    "cost":"10",
                    "timeout":"0",
                    "name": "dataAssetComputingServiceAgreement"
                }
            },
            "type": "compute",
            "index": 1,
            "serviceEndpoint": "http://mybrizo.org/api/v1/brizo/services/compute?pubKey=${"
                               "pubKey}&serviceId={serviceId}&algo={algo}&container={container}"
            
        },
        {
            "type": "metadata",
            "index": 2,
            "serviceEndpoint": "http://myaquarius.org/api/v1/provider/assets/metadata/{did}",
            "attributes": {
                "main": {
                    "name": "UK Weather information 2011",
                    "type": "dataset",
                    "dateCreated": "2012-10-10T17:00:00Z",
                    "datePublished": "2012-10-10T17:00:00Z",
                    "author": "Met Office",
                    "license": "CC-BY",
                    "files": [{
                        "index": 0,
                        "contentLength": "4535431",
                        "contentType": "text/csv",
                        "encoding": "UTF-8",
                        "compression": "zip",
                        "resourceId": "access-log2018-02-13-15-17-29-18386C502CAEA932"
                    }
                    ]
                },
                "encryptedFiles": "0xkasdhfkljhasdfkjasdhf",
                "curation": {
                    "rating": 0.0,
                    "numVotes": 0,
                    "schema": "Binary Votting",
                    "isListed": True
                },
                "additionalInformation": {
                    "description": "Weather information of UK including temperature and humidity",
                    "copyrightHolder": "Met Office",
                    "workExample": "stationId,latitude,longitude,datetime,temperature,"
                                   "humidity /n 423432fsd,51.509865,-0.118092,"
                                   "2011-01-01T10:55:11+00:00,7.2,68",
                    "inLanguage": "en",
                    "tags": ["weather", "uk", "2011", "temperature", "humidity"],
                    "updateFrequency": "yearly",
                    "structuredMarkup": [
                        {"uri": "http://skos.um.es/unescothes/C01194/jsonld",
                         "mediaType": "application/ld+json"},
                        {"uri": "http://skos.um.es/unescothes/C01194/turtle",
                         "mediaType": "text/turtle"}
                    ],
                    "links": [
                        {
                            "name": "Sample of Asset Data",
                            "type": "sample",
                            "url": "https://foo.com/sample.csv"
                        },
                        {
                            "name": "Data Format Definition",
                            "type": "format",
                            "url": "https://foo.com/sample2.csv"
                        }
                    ]

                }
            }
        }
    ]
}
json_update = {
    "@context": "https://w3id.org/future-method/v1",
    "created": "2016-02-08T16:02:20Z",
    "dataToken": "0xC7EC1970B09224B317c52d92f37F5e1E4fF6B687",
    "id": "did:op:112233445566778899",
    "publicKey": [
        {
            "id": "did:op:123456789abcdefghi#keys-1",
            "type": "RsaVerificationKey2018",
            "owner": "did:op:123456789abcdefghi",
            "publicKeyPem": "-----BEGIN PUBLIC KEY...END PUBLIC KEY-----\r\n"
        },
        {
            "id": "did:op:123456789abcdefghi#keys-2",
            "type": "Ed25519VerificationKey2018",
            "owner": "did:op:123456789abcdefghi",
            "publicKeyBase58": "H3C2AVvLMv6gmMNam3uVAjZpfkcJCwDwnZn6z3wXmqPV"
        }
    ],
    "authentication": [
        {
            "type": "RsaSignatureAuthentication2018",
            "publicKey": "did:op:123456789abcdefghi#keys-1"
        },
        {
            "type": "ieee2410Authentication2018",
            "publicKey": "did:op:123456789abcdefghi#keys-2"
        }
    ],
    "proof": {
        "type": "UUIDSignature",
        "created": "2016-02-08T16:02:20Z",
        "creator": "did:example:8uQhQMGzWxR8vw5P3UWH1ja",
        "signatureValue": "QNB13Y7Q9...1tzjn4w=="
    },
    "service": [
        {
            "attributes": {
                "main": {
                    "cost":"10",
                    "timeout":"0"              
                }
            },
            "type": "access",
            "index": 0,
            "serviceEndpoint": "http://mybrizo.org/api/v1/brizo/services/consume?pubKey=${"
                               "pubKey}&serviceId={serviceId}&url={url}"
        },
        {
            "type": "compute",
            "index": 1,
            "serviceEndpoint": "http://mybrizo.org/api/v1/brizo/services/compute?pubKey=${"
                               "pubKey}&serviceId={serviceId}&algo={algo}&container={container}",
            "attributes": {
                "main": {
                    "cost":"5",
                    "timeout":"0",
                    "name": "dataAssetComputingServiceAgreement"
                }
            }
        },
        {
            "type": "metadata",
            "index": 2,
            "serviceEndpoint": "http://myaquarius.org/api/v1/provider/assets/metadata/{did}",
            "attributes": {
                "main": {
                    "name": "UK Weather information 2012",
                    "type": "dataset",
                    "dateCreated": "2012-02-01T10:55:11Z",
                    "datePublished": "2012-02-01T10:55:11Z",
                    "author": "Met Office",
                    "license": "CC-BY",
                    "files": [{
                        "index": 0,
                        "contentLength": "4535431",
                        "contentType": "text/csv",
                        "encoding": "UTF-8",
                        "compression": "zip",
                        "resourceId": "access-log2018-02-13-15-17-29-18386C502CAEA932"
                    }]
                },
                "encryptedFiles": "0xkasdhfkljhasdfkjasdhf",
                "curation": {
                    "rating": 8.0,
                    "numVotes": 1,
                    "schema": "Binary Votting",
                    "isListed": True
                },
                "additionalInformation": {
                    "description": "Weather information of UK including temperature and humidity and white",
                    "copyrightHolder": "Met Office",
                    "workExample": "stationId,latitude,longitude,datetime,temperature,"
                                   "humidity /n 423432fsd,51.509865,-0.118092,"
                                   "2011-01-01T10:55:11+00:00,7.2,68",
                    "inLanguage": "en",
                    "tags": ["weather", "uk", "2011", "temperature", "humidity"],
                    "updateFrecuency": "yearly",
                    "structuredMarkup": [
                        {"uri": "http://skos.um.es/unescothes/C01194/jsonld",
                         "mediaType": "application/ld+json"},
                        {"uri": "http://skos.um.es/unescothes/C01194/turtle",
                         "mediaType": "text/turtle"}
                    ],
                    "links": [
                        {
                            "name": "Sample of Asset Data",
                            "type": "sample",
                            "url": "https://foo.com/sample.csv"
                        },
                        {
                            "name": "Data Format Definition",
                            "type": "format",
                            "url": "https://foo.com/sample2.csv"
                        }
                    ]
                }
            }
        }
    ]
}
json_valid = {
    "main": {
        "name": "10 Monkey Species Small",
        "dateCreated": "2012-02-01T10:55:11Z",
        "author": "Mario",
        "license": "CC0: Public Domain",
        "files": [
            {
                "index": 0,
                "contentType": "application/zip",
                "encoding": "UTF-8",
                "compression": "zip",
                "checksum": "2bf9d229d110d1976cdf85e9f3256c7f",
                "checksumType": "MD5",
                "contentLength": "12057507",
                "url": "https://s3.amazonaws.com/assets/training.zip"
            },
            {
                "index": 1,
                "contentType": "text/txt",
                "encoding": "UTF-8",
                "compression": "none",
                "checksum": "354d19c0733c47ef3a6cce5b633116b0",
                "checksumType": "MD5",
                "contentLength": "928",
                "url": "https://s3.amazonaws.com/datacommons/monkey_labels.txt"
            },
            {
                "index": 2,
                "contentType": "application/zip",
                "url": "https://s3.amazonaws.com/datacommons/validation.zip"
            }
        ],
        "type": "dataset",
    },
    "additionalInformation": {
        "description": "EXAMPLE ONLY ",
        "categories": [
            "image"
        ],
        "tags": [
            "image data",
            "classification",
            "animals"
        ],
        "workExample": "image path, id, label",
        "links": [
            {
                "name": "example model",
                "url": "https://drive.google.com/open?id=1uuz50RGiAW8YxRcWeQVgQglZpyAebgSM"
            },
            {
                "name": "example code",
                "type": "example code",
                "url": "https://github.com/slothkong/CNN_classification_10_monkey_species"
            },
            {
                "url": "https://s3.amazonaws.com/datacommons/links/discovery/n5151.jpg",
                "name": "n5151.jpg",
                "type": "discovery"
            },
            {
                "url": "https://s3.amazonaws.com/datacommons/links/sample/sample.zip",
                "name": "sample.zip",
                "type": "sample"
            }
        ],
        "copyrightHolder": "Unknown",
        "inLanguage": "en"
    }
}



ddo_event_sample = {
    "@context": "https://w3id.org/did/v1",
    "id": "did:op:ffa5037987b74fbab600d7515605146bb7babcb929c94c60ba93ac5ceda56775",
    "publicKey": [
      {
        "id": "did:op:ffa5037987b74fbab600d7515605146bb7babcb929c94c60ba93ac5ceda56775",
        "type": "EthereumECDSAKey",
        "owner": "0xBE5449a6A97aD46c8558A3356267Ee5D2731ab5e"
      }
    ],
    "authentication": [
      {
        "type": "RsaSignatureAuthentication2018",
        "publicKey": "did:op:ffa5037987b74fbab600d7515605146bb7babcb929c94c60ba93ac5ceda56775"
      }
    ],
    "service": [
      {
        "type": "metadata",
        "attributes": {
          "curation": {
            "rating": 0,
            "numVotes": 0,
            "isListed": True
          },
          "main": {
            "type": "dataset",
            "name": "Event DDO sample",
            "dateCreated": "2020-07-13T09:47:27Z",
            "author": "Met Office",
            "license": "CC-BY",
            "files": [
              {
                "checksum": "efb2c764274b745f5fc37f97c6b0e764",
                "contentLength": "4535431",
                "contentType": "text/csv",
                "encoding": "UTF-8",
                "compression": "zip",
                "index": 0
              }
            ],
            "datePublished": "2020-07-13T09:47:29Z"
          },
          "encryptedFiles": "0x045479dadbdb0ef593998d8f713f2a42585dc7415288326df5737fc3e2a42eee72d50748e820f495ac3abb0a5df99b299984adc2fec2c3883486b87855f739bbd83af73b9ef48726a9c7922badeeb5895ea392c840dccf6b1726de845baad5301087bba0bf447d518a46125b430409050e45a431aea4d3d08bdc97471d18eb0714960045efe65705b4c2670d5ca8e1428f4f53f2d2b11b21719098df8cf013d9aba5c8d1640a09e46a3fb68c87a0c02244caae754a4b6dffcde45e90b4921ae8d2fb84a8f771736860703d797d22ed6405d0cc03a0a5b7c6fab7c8e3ccf30ded364c28ee9a564db01dcdbe93b41a38ffcb0cdc66c43507c4a599e0cfa17561e4af41a6885076a399f41384aabf5c29b587b133b85cb4b879314c969c113d2fd3a552cad771fb752a21d325effe8b8814c603e17e01ff769c40a942c683d46918517a0210112ecd6a25a836d8778414ef24060ab821cbf9ea652a13fa35176fabdd"
        },
        "index": 0
      },
      {
        "type": "compute",
        "index": 1,
        "serviceEndpoint": "http://localhost:8030/api/v1/services/compute",
        "attributes": {
          "main": {
            "name": "dataAssetComputingService",
            "creator": "0xBE5449a6A97aD46c8558A3356267Ee5D2731ab5e",
            "datePublished": "2020-07-13T09:47:27Z",
            "cost": "1000",
            "timeout": 3600,
            "provider": {
              "type": "Azure",
              "description": "Compute service with 16gb ram for each node.",
              "environment": {
                "cluster": {
                  "type": "Kubernetes",
                  "url": "http://10.0.0.17/xxx"
                },
                "supportedServers": [
                  {
                    "image": "tensorflow/tensorflow",
                    "tag": "latest",
                    "checksum": "sha256:cb57ecfa6ebbefd8ffc7f75c0f00e57a7fa739578a429b6f72a0df19315deadc"
                  }
                ],
                "supportedContainers": [
                  {
                    "serverId": "1",
                    "serverType": "xlsize",
                    "cost": "50",
                    "cpu": "16",
                    "gpu": "0",
                    "memory": "128gb",
                    "disk": "160gb",
                    "maxExecutionTime": 86400
                  }
                ]
              }
            },
            "privacy": {
              "allowRawAlgorithm": False,
              "allowNetworkAccess": False,
              "trustedAlgorithms": []
            }
          }
        }
      },
      {
        "type": "access",
        "index": 2,
        "serviceEndpoint": "http://localhost:8030/api/v1/services/consume",
        "attributes": {
          "main": {
            "creator": "0xBE5449a6A97aD46c8558A3356267Ee5D2731ab5e",
            "datePublished": "2020-07-13T09:42:49Z",
            "cost": 10,
            "timeout": 0,
            "name": "dataAssetAccess"
          }
        }
      }
    ],
    "dataToken": "0x20e91598bb797eEd2C7D4431a274c2997D080f53",
    "created": "2020-07-13T09:47:29Z",
    "proof": {
      "created": "2020-07-13T09:47:29Z",
      "creator": "0xBE5449a6A97aD46c8558A3356267Ee5D2731ab5e",
      "type": "DDOIntegritySignature",
      "signatureValue": "0x48fa02b7227d09b2a4ae9d6bab9ed8bfc42b3c5bf105b8eba92f16f39842672e4605387cabdc56b4f42edf57879fb02ab9d4b1d699522d6b15c949b4a7a734f101"
    },
    "updated": "2020-07-13T09:47:29Z",
    "accessWhiteList": ["0x123","0x456"]
  }



algorithm_ddo_sample = {
  "@context": "https://w3id.org/did/v1",
  "authentication": [
    {
      "type": "RsaSignatureAuthentication2018",
      "publicKey": "did:op:0ebed8226ada17fde24b6bf2b95d27f8f05fcce09139ff5cec31f6d81a7cd2ea"
    }
  ],
  "created": "2019-02-08T08:13:49Z",
  "id": "did:op:0bc278fee025464f8012b811d1bce8e22094d0984e4e49139df5d5ff7a028bdf",
  "dataToken": "0xC7EC1970B09224B317c52d92f37F5e1E4fF6B687",
  "proof": {
    "created": "2019-02-08T08:13:41Z",
    "creator": "0x37BB53e3d293494DE59fBe1FF78500423dcFd43B",
    "signatureValue": "did:op:0bc278fee025464f8012b811d1bce8e22094d0984e4e49139df5d5ff7a028bdf",
    "type": "DDOIntegritySignature",
    "checksum": {
      "0": "0x52b5c93b82dd9e7ecc3d9fdf4755f7f69a54484941897dc517b4adfe3bbc3377",
      "1": "0x999999952b5c93b82dd9e7ecc3d9fdf4755f7f69a54484941897dc517b4adfe3"
    }
  },
  "publicKey": [
    {
      "id": "did:op:b6e2eb5eff1a093ced9826315d5a4ef6c5b5c8bd3c49890ee284231d7e1d0aaa#keys-1",
      "type": "RsaVerificationKey2018",
      "owner": "did:op:6027c1e7cbae06a91fce0557ee53195284825f56a7100be0c53cbf4391aa26cc",
      "publicKeyPem": "-----BEGIN PUBLIC KEY...END PUBLIC KEY-----\r\n"
    },
    {
      "id": "did:op:b6e2eb5eff1a093ced9826315d5a4ef6c5b5c8bd3c49890ee284231d7e1d0aaa#keys-2",
      "type": "Ed25519VerificationKey2018",
      "owner": "did:op:4c27a254e607cdf91a1206480e7eb8c74856102316c1a462277d4f21c02373b6",
      "publicKeyBase58": "H3C2AVvLMv6gmMNam3uVAjZpfkcJCwDwnZn6z3wXmqPV"
    },
    {
      "id": "did:op:b6e2eb5eff1a093ced9826315d5a4ef6c5b5c8bd3c49890ee284231d7e1d0aaa#keys-3",
      "type": "RsaPublicKeyExchangeKey2018",
      "owner": "did:op:5f6b885202ffb9643874be529302eb00d55e226959f1fbacaeda592c5b5c9484",
      "publicKeyPem": "-----BEGIN PUBLIC KEY...END PUBLIC KEY-----\r\n"
    }
  ],
  "verifiableCredential": [
    {
      "@context": [
        "https://www.w3.org/2018/credentials/v1",
        "https://www.w3.org/2018/credentials/examples/v1"
      ],
      "id": "1872",
      "type": [
        "read",
        "update",
        "deactivate"
      ],
      "issuer": "0x610D9314EDF2ced7681BA1633C33fdb8cF365a12",
      "issuanceDate": "2019-01-01T19:73:24Z",
      "credentialSubject": {
        "id": "0x89328493849328493284932"
      },
      "proof": {
        "type": "RsaSignature2018",
        "created": "2019-01-01T19:73:24Z",
        "proofPurpose": "assertionMethod",
        "signatureValue": "ABCJSDAO23...1tzjn4w=="
      }
    }
  ],
  "service": [
    {
      "index": 0,
      "serviceEndpoint": "http://localhost:5000/api/v1/aquarius/assets/ddo/{did}",
      "type": "metadata",
      "attributes": {
        "main": {
          "author": "John Doe",
          "dateCreated": "2019-02-08T08:13:49Z",
          "license": "CC-BY",
          "name": "My super algorithm",
          "type": "algorithm",
          "algorithm": {
            "language": "scala",
            "format": "docker-image",
            "version": "0.1",
            "container": {
              "entrypoint": "node $ALGO",
              "image": "node",
              "tag": "10"
            }
          },
          "files": [
            {
              "name": "build_model",
              "url": "https://raw.githubusercontent.com/oceanprotocol/test-algorithm/master/javascript/algo.js",
              "index": 0,
              "checksum": "efb2c764274b745f5fc37f97c6b0e761",
              "contentLength": "4535431",
              "contentType": "text/plain",
              "encoding": "UTF-8",
              "compression": "zip"
            }
          ]
        },
        "additionalInformation": {
          "description": "Workflow to aggregate weather information",
          "tags": [
            "weather",
            "uk",
            "2011",
            "workflow",
            "aggregation"
          ],
          "copyrightHolder": "John Doe"
        }
      }
    }
  ]
}



#  Copyright 2018 Ocean Protocol Foundation
#  SPDX-License-Identifier: Apache-2.0
import copy
import json
import logging
from datetime import datetime

DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
DATETIME_FORMAT_NO_Z = '%Y-%m-%dT%H:%M:%S'

logger = logging.getLogger('aquarius')


def sanitize_record(data_record):
    if '_id' in data_record:
        data_record.pop('_id')
    if 'event' in data_record:
        data_record.pop('event')

    return json.dumps(data_record, default=datetime_converter)


def make_paginate_response(query_list_result, search_model):
    total = query_list_result[1]
    offset = search_model.offset
    result = dict()
    result['results'] = query_list_result[0]
    result['page'] = search_model.page

    result['total_pages'] = int(total / offset) + int(total % offset > 0)
    result['total_results'] = total
    return result


def datetime_converter(o):
    if isinstance(o, datetime):
        return o.strftime(DATETIME_FORMAT)


def format_timestamp(timestamp):
    try:
        return f'{datetime.strptime(timestamp, DATETIME_FORMAT).replace(microsecond=0).isoformat()}Z'
    except Exception:
        return f'{datetime.strptime(timestamp, DATETIME_FORMAT_NO_Z).replace(microsecond=0).isoformat()}Z'


def get_timestamp():
    """Return the current system timestamp."""
    return f'{datetime.utcnow().replace(microsecond=0).isoformat()}Z'


def get_curation_metadata(services):
    return get_metadata_from_services(services)['attributes']['curation']


def get_main_metadata(services):
    return get_metadata_from_services(services)['attributes']['main']


def get_metadata_from_services(services):
    for service in services:
        if service['type'] == 'metadata':
            return service


def reorder_services_list(services):
    result = []
    for service in services:
        if service['type'] == 'metadata':
            result.append(service)

    for service in services:
        if service['type'] != 'metadata':
            result.append(service)

    return result


def init_new_ddo(data):
    _record = copy.deepcopy(data)
    _record['created'] = format_timestamp(data['created'])
    _record['updated'] = _record['created']
    if 'accessWhiteList' not in data:
        _record['accessWhiteList'] = []
    else:
        if not isinstance(data['accessWhiteList'], list):
            _record['accessWhiteList'] = []
        else:
            _record['accessWhiteList'] = data['accessWhiteList']

    for service in _record['service']:
        if service['type'] == 'metadata':
            samain = service['attributes']['main']
            samain['dateCreated'] = format_timestamp(samain['dateCreated'])
            samain['datePublished'] = get_timestamp()

            curation = dict()
            curation['rating'] = 0.00
            curation['numVotes'] = 0
            curation['isListed'] = True
            service['attributes']['curation'] = curation
    _record['service'] = reorder_services_list(_record['service'])
    return _record


def validate_date_format(date):
    try:
        datetime.strptime(date, DATETIME_FORMAT)
        return None, None
    except Exception as e:
        logging.error(f'validate_date_format: {str(e)}')
        return f"Incorrect data format, should be '{DATETIME_FORMAT}'", 400


def check_no_urls_in_files(main, method):
    if 'files' in main:
        for file in main['files']:
            if 'url' in file:
                logger.error(
                    '%s request failed: url is not allowed in files ' % method)
                return '%s request failed: url is not allowed in files ' % method, 400
    return None, None


def check_required_attributes(required_attributes, data, method):
    assert isinstance(
        data, dict), 'invalid `body` type, should already formatted into a dict.'
    logger.info('got %s request: %s' % (method, data))
    if not data:
        logger.error('%s request failed: data is empty.' % method)
        logger.error('%s request failed: data is empty.' % method)
        return 'payload seems empty.', 400

    for attr in required_attributes:
        if attr not in data:
            logger.error(
                '%s request failed: required attr %s missing.' % (method, attr))
            return '"%s" is required in the call to %s' % (attr, method), 400

    return None, None


def list_errors(list_errors_function, data):
    error_list = list()
    for err in list_errors_function(data):
        stack_path = list(err[1].relative_path)
        stack_path = [str(p) for p in stack_path]
        this_err_response = {
            'path': "/".join(stack_path), 'message': err[1].message}
        error_list.append(this_err_response)
    return error_list


def validate_data(data, method):
    required_attributes = ['@context', 'created', 'id', 'publicKey', 'authentication', 'proof',
                           'service', 'dataToken']

    msg, status = check_required_attributes(
        required_attributes, data, method)
    if msg:
        return msg, status
    msg, status = check_no_urls_in_files(
        get_main_metadata(data['service']), method)
    if msg:
        return msg, status
    msg, status = validate_date_format(data['created'])
    if status:
        return msg, status
    return None, None


def get_sender_from_txid(web3, txid):
    transaction = web3.eth.getTransaction(txid)
    if transaction is not None:
        return transaction['from']
    return None
